/*
 * sys_unlink.c - file unlink
 * 
 * Copyright (c) 2008,2009,2010,2011,2012 Ghassan Almaless
 * Copyright (c) 2011,2012 UPMC Sorbonne Universites
 *
 * This file is part of ALMOS-kernel.
 *
 * ALMOS-kernel is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.0 of the License.
 *
 * ALMOS-kernel is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ALMOS-kernel; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <hal_types.h>
#include <hal_uspace.h>
#include <vfs.h>
#include <process.h>
#include <thread.h>
#include <printk.h>

//////////////////////////////////
int sys_unlink ( char * pathname )
{
	error_t   error;
    uint32_t  length;
    char      kbuf[CONFIG_VFS_MAX_PATH_LENGTH];

	thread_t     * this     = CURRENT_THREAD;
	process_t    * process  = this->process;

    // get pathname length
    length = hal_strlen_from_uspace( pathname );

    if( length >= CONFIG_VFS_MAX_PATH_LENGTH )
    {
        printk("\n[ERROR] in %s : pathname too long\n", __FUNCTION__ );
		this->errno = ENFILE;
        return -1;
    }
 
	// get pathname copy in kernel space
    hal_copy_from_uspace( kbuf, pathname, length );

    // get cluster and local pointer on reference process
    xptr_t      ref_xp  = process->ref_xp;
    process_t * ref_ptr = (process_t *)GET_PTR( ref_xp );
    cxy_t       ref_cxy = GET_CXY( ref_xp );

    // get the cwd lock in read mode from reference process
	remote_rwlock_rd_lock( XPTR( ref_cxy , &ref_ptr->cwd_lock ) );

    // get extended pointer on cwd inode
    xptr_t cwd_xp = hal_remote_lwd( XPTR( ref_cxy , &ref_ptr->vfs_cwd_xp ) );

    // call relevant VFS function
    error  = vfs_unlink( cwd_xp , kbuf );

    // release the cwd lock in reference process
	remote_rwlock_rd_unlock( XPTR( ref_cxy , &ref_ptr->cwd_lock ) );

	if( error )
	{
        printk("\n[ERROR] in %s : cannot unlink file/dir %s\n",
               __FUNCTION__ , pathname );
		this->errno = ENFILE;
	    return -1;
	}

	return 0;

} // end sys_unlink()
